/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import org.nfunk.jep.ParseException;

/**
 * A convenient way to implement series, that is, defining them with a string
 * that will be parsed.
 * 
 * @see ParsedFunction
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class ParsedSeries extends ParsedSuccession {
    /** Minimum index from which begin calculating the series. */
    private int min = 0;

    /** Default constructor. */
    ParsedSeries() {
        super();
        this.min = 0;
    }

    /**
     * Standard constructor.
     * 
     * @see ParsedFunction
     * @see org.nfunk.jep.JEP
     * @param min
     *            the minimum index of the series
     * @param s
     *            the string to be parsed
     * @throws ParseException
     *             if the string couldn't be parsed correctly
     */
    public ParsedSeries(int min, String s) throws ParseException {
        super(s);
        this.min = min;
    }

    /**
     * Returns the y value corresponding to the specified index n.
     * 
     * @param n
     *            the non-negative integer index
     * @return the value of the succession
     */
    public double f(int n) {
        double result;
        // Little performance trick for sequential calculations
        int lastAskedX = this.min - 1;
        double lastAnsweredY = 0;
        
        try {
            if (n < this.min) {
                throw new ArithmeticException();
            }
            if (n == this.min) {
                result = super.f(n);
            } else {

                result = this.internalFunction.f(n);

                // "cached" implementation
                if (n - 1 == lastAskedX) {
                    result += lastAnsweredY;
                }
                // no cache, recursively call :(
                else {
                    result += f(n - 1);
                }

                // update the cache
                lastAskedX = n;
                lastAnsweredY = result;
            }
        } catch (ArithmeticException e) {
            result = Double.NaN;
        }
        return result;
    }
}